import click
import pyperclip

from wid.web.url import Url
from wid.web.bot.webcrawler import WebCrawler
from wid.web.bot.instructions import Instructions

from wid.file.utils import import_module




@click.command()
@click.option('--url', '-u', default=None, help='Url of the starting website for the web crawler. Contents of the clipboard are used if none is provided.')
@click.option('--instructions', '-i', default=None, help='Implementation of Instructions abstract class used by the WebCrawler.')
@click.option('--target-dir', '-t', default=None, help='Target directory used to store images.')
def main(url, instructions, target_dir):
    
    """ Python web crawler for extracting and saving images from websites. """    

    url = pyperclip.paste() if url is None else url
    target_url = Url(url)
    target_dir = target_dir if target_dir is not None else './wid-images'
    
    if target_url is None or not target_url.is_valid():
        click.echo('Error: Target URL is invalid.')    
        click.echo(main.get_help(click.Context(main)))
        exit()
    
    try:
        instructions_module = import_module(instructions)
        webcrawler_instructions = instructions_module.__InstructionClass__()
        
    except Exception:
        raise ImportError('Failed to import and instantiate user-defined Instructions class from {}.'.format(instructions))

    start_crawler(webcrawler_instructions, target_url, target_dir)



def start_crawler(instructions: Instructions, starting_url: Url, target_dir: str) -> None:
    
    if not starting_url.is_valid():
        raise ValueError('ValueError: Invalid target URL: \'{}\'.'.format(starting_url))
    
    click.echo('Starting web crawler on page {}.'.format(starting_url))
    
    webcrawler = WebCrawler(instructions, starting_url, target_dir)
    webcrawler.crawl()

    click.echo('Done.')
    
    

if __name__ == "__main__":
    
    main()


from typing import List

import abc

from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.remote.webelement import WebElement

from wid.web.url import Url




__all__ = ['Instructions']


class Instructions(abc.ABC):
    
    @abc.abstractmethod
    def validate(self, webdriver: WebDriver, url: Url) -> bool:
        """
        (Optional) Function that instructs webdriver how to pass validation for web page
        given by the url. User implemented instructions on how to bybass age validation,
        login, captcha etc. It's necessary to first load the URL and then take steps
        necessary to pass validation.
        
        Parameters
        -------------------------------------------------------------------------
            webdriver : WebDriver
                initiated Selenium webdriver instance
            urk : Url
                starting URL where the WebCrawler need to pass validation
            
        Returns
        -------------------------------------------------------------------------
            success : bool
                return true of the validation succeeded, false otherwise
        """    
        raise NotImplementedError
    
    @abc.abstractmethod
    def next_step(self, webdriver: WebDriver) -> List[Url]:
        """
        Function that parses the webdriver.current_url and finds a list of URLs that are
        to by added to WebCrawler's URL queue. URLs are processed in FIFO order.
        
        Parameters
        -------------------------------------------------------------------------
            webdriver : WebDriver
                initiated Selenium webdriver instance with current_url loaded

        Returns
        -------------------------------------------------------------------------
            urls : List[Url]
                list of URLs found on webdriver.curren_url
        """            
        raise NotImplementedError
    
    @abc.abstractmethod
    def find_image_elements(self, webdriver: WebDriver) -> List[WebElement]:
        """
        Function that parses the webdriver.current_url and finds a list of image
        elements with source URLs that are to be downloaded by the WebCrawler.
        
        Parameters
        -------------------------------------------------------------------------
            webdriver : WebDriver
                initiated Selenium webdriver instance with current_url loaded

        Returns
        -------------------------------------------------------------------------
            image_elements : List[WebElement]
                list of WebElements containing desired images and their source URLs
        """   
        raise NotImplementedError
    
    
""" 
User-defined constant with the class name of the user's subclass of Instructions.
Used to dynamically import and instantiate the subclass of Instructions.

Must be defined by the user e.g. __InstructionClass__ = MyWebcrawlerInstructions
"""
__InstructionClass__ = Instructions
    


from typing import List

from selenium import webdriver
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.chrome.options import Options as ChromeOptions

from selenium.common.exceptions import WebDriverException

from webdriver_manager.chrome import ChromeDriverManager

from wid.web.url import Url




__all__ = ['initialize_webdriver', 'find_image_urls', 'get_element_src', 'get_page_source']


def initialize_webdriver() -> WebDriver:
    """
    Function initialized a headless Chrome webdriver instance with suppressed
    warnings/errors.
    
    Returns
    -------------------------------------------------------------------------
        driver : selenium.remote.webdriver.WebDriver
           webdriver instance
    """           
    chrome_driver_path = ChromeDriverManager().install()

    chrome_options = ChromeOptions()
    chrome_options.add_argument('--headless')
    chrome_options.add_argument('--window-size=1920,1080')
    chrome_options.add_argument('--log-level=3')
    
    return webdriver.Chrome(chrome_driver_path,
                            chrome_options=chrome_options)
    
    

def find_image_urls(target_url: Url) -> List[Url]:
    """
    Function searches the web page referenced by target_url for images. The function
    looks through the DOM tree structure for 'img' elements and return a list of URLs
    referencing each image.
    
    Parameters
    -------------------------------------------------------------------------
        target_url : web.url.Url
            url of the target website
    Returns
    -------------------------------------------------------------------------
        image_list : List[web.url.Url]
           list of URLs to images found on the website
    """      
    if not target_url.is_valid():
        print('ValueError: Invalid target URL: \'{}\'.'.format(target_url))
        raise ValueError
    
    driver = initialize_webdriver()
    
    try:
        driver.get(target_url)
      
        image_elements = driver.find_elements_by_tag_name('img')
        image_urls = [get_element_src(e, target_url) for e in image_elements]

        driver.close()

    except WebDriverException as e:
        print('Error: Could not resolve site \'{}\'.'.format(target_url))
        raise e

    finally:
        driver.quit()

    return image_urls



def get_element_src(element: WebElement, url: Url) -> Url:
    """
    Function returns the source URL for given element. If the source URL is
    relative then the function takes base_name from the provided URL and prepends
    it to the relative URL.
    
    Parameters
    -------------------------------------------------------------------------
        element : selenium.webdriver.remote.webelement.WebElement
            HTML element
        url : web.url.Url
            url of the web page where the element is located
            
    Returns
    -------------------------------------------------------------------------
        url : web.url.Url
           source URL of given element
    """     
    # Elements with absolute URLs
    if element.get_attribute('src'):
        url_string = element.get_attribute('src')
        return Url(url_string)
        
    # Elements relative URLs -> prepend base address if needed
    elif element.get_attribute('data-src'):
        
        url_string = element.get_attribute('data-src').strip()
        src = Url(url_string)

        if src.is_valid(): 
            return src
        
        else:
            url_string = url.get_base_url().strip() + element.get_attribute('data-src')
            return Url(url_string)
    
    # Element has neither attribute -> missing a link?
    else:
        return None
    
    

def get_page_source(target_url: Url) -> str:
    """
    Function returns the full HTML source code of the web page referenced by
    target_url.
    
    Parameters
    -------------------------------------------------------------------------
        url : web.url.Url
            url of a web page
            
    Returns
    -------------------------------------------------------------------------
        page_source : str
           string containing the source code of given web page
    """   
    if not target_url.is_valid():
        print('ValueError: Invalid target URL: \'{}\'.'.format(target_url))
        raise ValueError        
    
    driver = initialize_webdriver()
    
    try:
        driver.get(target_url)
        
        page_source = driver.page_source
        
        driver.close()
                
    except WebDriverException as e:
        print('Error: Could not resolve site \'{}\'.'.format(target_url))
        raise e
    
    finally:
        driver.quit()
        
    return page_source



from typing import List, Callable

import os
from collections import deque

from selenium.common.exceptions import WebDriverException

import wid.web.img.save
import wid.web.img.scrape

from wid.web.url import Url
from wid.web.bot.instructions import Instructions


# TODO: URL normalization / canonicalization
# E.g. example.com/, example.com, example.com/index.html, example.com/index.php -> one single URL
# => Use url-normalize library for Python?


__all__  = ['WebCrawler']


class WebCrawler():
    """ 
    Class WebCrawler is used to parse URLs, find and save desired images and
    search for next set of URLs to explore. WebCrawler will continue to function
    while there are URLs in it's url_queue.
    
    Attributes
    -------------------------------------------------------------------------
        webdriver : selenium.webdriver.remote.webdriver.WebDriver
            instance of Selenium's Chrome webdriver 
        instructions : web.bot.Instructions
            instance of user-defined subclass of web crawler Instructions
        url_queue : deque[Url]
            FIFO queue of URLs that are explored by the web crawler
        url_visited : set[Url]
            set of URLs already visited by the web crawler
        target_dir : str
            directory path for saving image files
        
    Methods
    -------------------------------------------------------------------------
        crawl(self) -> None
            starts the WebCrawler, navigates and scrapes web pages and downloads image files
            until there are no URLs left in url_queue.
        _navigate_url(self, target_url: Url) -> None
            uses WebDriver to load target_url and adds it to url_visited set
        _get_next_url_list(self, current_url: Url, step_function: Callable[[Url], List[Url]]=None) -> List[Url]
            uses Instructions.next_step() or step_function() to get a set of not-visited URLs
        _find_image_urls(self, visited_page_url: Url) -> List[Url]
            finds src of desired image elements using Instructions.find_image_elements() on the currently loaded web page
        _download_images(self, image_urls: List[Url]) -> None
            downloads a list of images and saves them in location self.target_dir        
    """    
    def __init__(self, instructions: Instructions, starting_url: Url, target_dir: str='./wid-images') -> None:

        if not starting_url.is_valid():
            print('ValueError: Invalid target URL: \'{}\'.'.format(starting_url))
            raise ValueError
        
        self.webdriver = None
        self.instructions = instructions
        
        self.url_queue = deque([starting_url])
        self.url_visited = set()

        self.target_dir = target_dir

                
  
    def crawl(self) -> None:
        
        self.webdriver = wid.web.img.scrape.initialize_webdriver()
        self.instructions.validate(self.webdriver, self.url_queue[0])
        
        while len(self.url_queue) > 0:
            target_url = self.url_queue.popleft()
            print('Parsing web page \'{}\'.'.format(target_url))
            
            try:
                self._navigate_url(target_url)
                image_urls = self._find_image_urls(target_url)
                self._download_images(image_urls)

                next_urls = self._get_next_url_list()
                self.url_queue.extend(next_urls)
                   
            except WebDriverException:
                # Failed to load next web page -> continue with next in queue 
                continue
            
        self.webdriver.quit()            
            
            
            
    def _navigate_url(self, target_url: Url) -> None:
        # Add both target_url and url loaded by the webdriver in case of inconsistencies
        # -> avoid endless loop of navigation
        if self.webdriver is None:
            raise ValueError('WebDriver is not initialized in WebCrawler.')        
        
        self.webdriver.get(target_url)
        self.url_visited.add(target_url)
        self.url_visited.add(self.webdriver.current_url)
        
        
        
    def _get_next_url_list(self, step_function: Callable[[Url], List[Url]]=None) -> List[Url]:
        
        if step_function is None:
            next_url_list = self.instructions.next_step(self.webdriver)
        else:
            step_function(self.webdriver)

        valid_url_list = [url for url in next_url_list if url.is_valid()]
        return [url for url in valid_url_list if url not in self.url_visited]

    
    
    def _find_image_urls(self, visited_page_url: Url) -> List[Url]:
        if self.webdriver is None:
            raise ValueError('WebDriver is not initialized in WebCrawler.')    
        
        image_elements = self.instructions.find_image_elements(self.webdriver)
        return [wid.web.img.scrape.get_element_src(e, visited_page_url) for e in image_elements]
    
    
    
    def _download_images(self, image_urls: List[Url]) -> None:
        if self.webdriver is None:
            raise ValueError('WebDriver is not initialized in WebCrawler.') 

        url_dir_name = self.webdriver.current_url.split('/')[-2]
        dir_path = os.path.join(self.target_dir, url_dir_name)
        wid.web.img.save.save_images(image_urls, dir_path)
        
        

    


from types import ModuleType

import os
import sys
import importlib




__all__ = ['create_dir', 'import_module']


def create_dir(path_to_dir: str) -> None:
    """
    Function creates all nonexistent directories on the path_to_dir file path.
    
    Parameters
    -------------------------------------------------------------------------
        path_to_dir : str
            save file location for the image
            
    Returns
    -------------------------------------------------------------------------
        None
    """  
    if not os.path.isdir(path_to_dir):
        
        try:
            os.makedirs(path_to_dir)
        
        except Exception:
            print('Error: Failed to create target_dir \'{}\'.'.format(path_to_dir))
            
            
            
def import_module(path_to_file: str) -> ModuleType:
    """
    Function dynamically imports a Python module from source file located in path_to_file. 
    The new module is added into sys.modules dictionary. Name of the module is the basename
    of the path_to_file.
    
    Parameters
    -------------------------------------------------------------------------
        path_to_dir : str
            location of the module's source file
            
    Returns
    -------------------------------------------------------------------------
        module : ModuleType
            instance of a Python module class
    """      
    if not os.path.exists(path_to_file):
        raise FileNotFoundError('Error: File not found at location {}.'.format(path_to_file))
    
    module_name = os.path.basename(path_to_file)
    
    try:
        spec = importlib.util.spec_from_file_location(module_name, path_to_file)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)
        sys.modules[module_name] = module
        return module
    
    except Exception:
        raise ImportError('Failed to import module {} from {}.'.format(module_name, path_to_file))
    


from typing import List

import os
import requests
from urllib.request import urlretrieve

import wid.file.utils
from wid.web.url import Url




__all__ = ['save_images', 'download_image']

             
def save_images(url_list: List[Url], path_to_dir: str) -> None:
    """
    Function saves a set of images from a list of URLs to a location on disk.
    Each image is saved using web.img.save.download_image() function. 
    If there are any missing directories on the path_to_dir, they are created
    using wid.file.utils.create_dir().
    
    
    Parameters
    -------------------------------------------------------------------------
        url_list : List[web.url.Url]
            list of URL references to images
        path_to_dir : str
            save file location for the images in url_list
            
    Returns
    -------------------------------------------------------------------------
        None
    """   
    wid.file.utils.create_dir(path_to_dir)
    
    for url in url_list:
        
        try:
            print('Downloading image from \'{}\'.'.format(url))
            download_image(url, path_to_dir)
        
        except Exception as e:
            raise e
            print('Could not retrieve image from \'{}\'.'.format(url))



def download_image(url: Url, path_to_dir: str = './') -> None:
    """
    Function downloads a single image pointed to by a URL and save it to a
    location on the disk. If a location is not provided by the path_to_dir
    parameter, then working directory './' is used by default.
    
    Name of the image file is the basename of the path portion of the URL.
    It is the string after last '/' symbol e.g.:
    https://duckduckgo.com/assets/icons/header/reddit.svg -> reddit.svg
    
    Parameters
    -------------------------------------------------------------------------
        url : web.url.Url
            URL reference to the image
        path_to_dir : str
            save file location for the image
            
    Returns
    -------------------------------------------------------------------------
        None
    """   
    if url is not None:
        img_path = url.path
        img_basename = os.path.basename(img_path)
        save_file_path = os.path.join(path_to_dir, img_basename) 
        
        try:
            urlretrieve(url, save_file_path)
        
        except Exception:
            with requests.get(url, stream=True) as r:
                with open(save_file_path, 'wb') as f:
                    f.write(r.content)
                

            


import os

import click
import pyperclip

from wid.web.url import Url

import wid.web.img.save
import wid.web.img.scrape

import wid.file.utils




@click.command()
@click.option('--url', '-u', default=None, help='Url of the website containing desired images. Contents of the clipboard are used if none is provided.')
@click.option('--target-dir', '-t', default=None, help='Target directory used to store images.')
@click.option('--img-regex', '-r', default=None, help='Regex for finding specific subset of images on the website.')

@click.option('--img-info', '-i', is_flag=True, help='Option to find and print all image URLs on the website.')
@click.option('--page-source', '-p', is_flag=True, help='Option to get the source code of target website and print it (save it).')
def main(url: str, target_dir: str, img_regex: str, img_info: bool, page_source: bool) -> None:
    
    """ Python script for extracting and saving images from websites. """
        
    url_string = pyperclip.paste() if url is None else url
    target_url = Url(url_string)
    
    if target_url is None or not target_url.is_valid():
        click.echo('Error: Target URL is invalid.')    
        click.echo(main.get_help(click.Context(main)))
        exit()
    
    if img_info:
        get_img_info(target_url, img_regex)
        
    if page_source:
        target_dir = target_dir if target_dir is not None else './wid-page-source'
        get_page_source(target_url, target_dir)
       
    if not img_info and not page_source:
        target_dir = target_dir if target_dir is not None else './wid-images'
        download_images(target_url, target_dir, img_regex)
        


def get_img_info(target_url: Url, img_regex: str) -> None:

    click.echo('Starting to parse {}...'.format(target_url))

    try:
        # Find URLs for all images on target website   
        image_urls = wid.web.img.scrape.find_image_urls(target_url)
        
        # Filter image links
        if img_regex is not None:
            image_urls = Url.filter_url_list(img_regex, image_urls)

        # Print the URLs of images 
        print('Images found:')
        for url in image_urls:
            if url is not None:
                print(url)
            
    except Exception as e:
        click.echo('Failed.')        
        raise e
        
    else:
        click.echo('Done.')
        
        
        
def get_page_source(target_url: Url, target_dir: str) -> None:

    click.echo('Attempting to open {}...'.format(target_url))

    try:
        # Load the page and get its source code
        page_src = wid.web.img.scrape.get_page_source(target_url)
            
        # Save the source code to a file (implement in save.py)
        file_name = target_url.netloc + '.txt'
        save_file_path = os.path.join(target_dir, file_name).replace('\\', '/')
        wid.file.utils.create_dir(target_dir)

        with open(save_file_path, 'w', encoding='utf8') as f:
            f.write(page_src)
            
    except Exception:
        click.echo('Failed.')        
        
    else:
        click.echo('Page source code saved.')
        
        

def download_images(target_url: Url, target_dir: str, img_regex: str) -> None:
    
    click.echo('Starting to parse {}...'.format(target_url))
        
    try:
        # Find URLs for all images on target website   
        image_urls = wid.web.img.scrape.find_image_urls(target_url)
        
        # Filter images
        if img_regex is not None:
            image_urls = Url.filter_url_list(img_regex, image_urls)
        
        # Store images in target location
        wid.web.img.save.save_images(image_urls, target_dir)    
    
    except Exception:
        click.echo('Failed.')        
    
    else:
        click.echo('Done.')
        


if __name__ == "__main__":
    
    main()


from __future__ import annotations

from typing import List

import re
import functools

import urllib.parse




__all__ = ['Url']


class Url(str):
    """ 
    Class Url represents the individual part of the URL accoring to the
    standard RFC_1808 (https://www.rfc-editor.org/rfc/rfc1808.html). 
    
    URL structure - <scheme>://<netloc>/<path>;<params>?<query>#<fragment>
    
    Attributes
    -------------------------------------------------------------------------
        scheme : str
            scheme name, as per Section 2.1 of RFC 1738
        netloc : str
            network location and login information, as per Section 3.1 of RFC 1738
        path : str
            URL path, as per Section 3.1 of RFC 1738
        params : str
            object parameters, e.g., ";type=a", as in Section 3.2.2 of RFC 1738
        query : str
            query information, as per Section 3.3 of RFC 1738
        fragment : str
            fragment identifier
        
    Methods
    -------------------------------------------------------------------------
        is_valid(self) -> bool
            returns True if the instance is a valid URL, False otherwise
        get_base_url(self) -> str
            returns the base name of the URL - scheme + netlock
        match(self, re.Pattern) -> bool
            returns True if the URL matches the structure of a URL, False otherwise
    
    Static Methods
    -------------------------------------------------------------------------
        filter_url_list(List[Url], str) -> List[Url]
            returns a subset of URLs from the input list that match given regex
    
    """
    
    
    _url_regex = ("((http|https|ftp)://)(www.)?" +
                "[a-zA-Z0-9@:%._\\+~#?&//=]" +
                "{2,256}\\.[a-z]" +
                "{2,6}\\b([-a-zA-Z0-9@:%" +
                "._\\+~#?&//=]*)")
    
    _url_pattern = re.compile(_url_regex)

    

    def __init__(self, url_string: str, default_scheme: str='https://') -> None:
        """
        Constructs an instance of the class Url. Doesn't work for relative URIs.
        
        Uses urllib to parse the input url_string. This library requires the
        URL to have a scheme to function properly. Constructor automatically
        prepends default_scheme to the url_string if none is present.
        
        Parameters
        -------------------------------------------------------------------------
            url_string : str
                input url string
            default_scheme : str
                default scheme to be used if there is no scheme in the input url
        """
        
        self.url_string = url_string if '//' in url_string else default_scheme + url_string
                
        parsed_url = urllib.parse.urlparse(self.url_string)
        
        self.scheme = parsed_url.scheme
        self.netloc = parsed_url.netloc
        self.path = parsed_url.path
        self.params = parsed_url.params
        self.query = parsed_url.query
        self.fragment = parsed_url.fragment
        
            
    
    def is_valid(self) -> bool:  
        """
        Function checks the validity of the URL represented by an instance of the class.
        
        Returns
        -------------------------------------------------------------------------
            is_valid : bool
                True if the _url_pattern matches represented URL, False otherwise
        """
        if not self._url_pattern.match(self.__str__()):
            return False  
        
        else:
            return True
        
        
        
    def get_base_url(self) -> str:
        """
        Function returns the base name of the URL. Base name has the following form:
        
        <scheme>://<netloc>
        
        Returns
        -------------------------------------------------------------------------
            base_url : str
                string representing the base name of the URL
        """        
        if self.scheme is not None and self.scheme != '':
            return self.scheme + '://' + self.netloc
        
        else:
            return self.netloc
    
    
    
    def match(self, pattern: re.Pattern) -> bool:
        """
        Function checks if the URL represented by an instance of the class matches
        the provided pattern.
        
        Parameters
        -------------------------------------------------------------------------
            pattern : re.Pattern
                precompiled pattern from a regex expression used to match the URL
            
        Returns
        -------------------------------------------------------------------------
            is_match : bool
                True if URL matches the patter, False otherwise
        """   
        
        if pattern.match(self.url_string):
            return True
        
        else:
            return False
        
        
    
    def __eq__(self, __x: object) -> bool:
        
        return super().__eq__(__x)
        
     
        
    def __hash__(self) -> int:
        return super().__hash__()
    
    
        
    def __repr__(self) -> str:
        
        return self.__str__()
    
            
            
    def __str__(self) -> str:
        
        return urllib.parse.urlunparse((self.scheme,
                                        self.netloc,
                                        self.path,
                                        self.params,
                                        self.query,
                                        self.fragment))
            
    
    
    @functools.singledispatchmethod
    @classmethod
    def filter_url_list(cls, url_filter, url_list: List[Url]) -> List[Url]:
        """
        Function filters a list of instances of Url class.
        
        Parameters
        -------------------------------------------------------------------------
            url_filter : str or re.Pattern
                filter used on the list of URLs, can be a regex string or a re.Pattern instance
            
        Returns
        -------------------------------------------------------------------------
            list : List[Url]
                list of Url class instances that match the provided regex expression
        """     
        raise NotImplementedError('TypeError:Filter argument is not a supported type.')
              
    @filter_url_list.register(str)
    @classmethod
    def _(cls, url_filter, url_list: List[Url]) -> List[Url]:
        pattern = re.compile(url_filter)
        filter_func = lambda x: re.match(pattern, x)
        return list(filter(filter_func, url_list))
    
    @filter_url_list.register(re.Pattern)
    @classmethod
    def _(cls, url_filter, url_list: List[Url]) -> List[Url]:
        filter_func = lambda x: re.match(url_filter, x)
        return list(filter(filter_func, url_list))
    
    




import setuptools

with open('ReadMe.md', 'r') as f:
    long_description = f.read()
    
with open('requirements.txt', 'r', encoding='UTF-16') as f:
    required = f.readlines()


setuptools.setup(
    name="webpage-image-downloader",
    version="0.1.2",
    
    author="A-Bak",
    author_email="adam.bak753@gmail.com",
    
    description="Tool for extracting and saving specific images from websites.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords='webscraping webcrawler image img downloader selenium-python',
    url='https://github.com/A-Bak/webpage-img-downloader',
    
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    install_requires=required,
    
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    
    entry_points={
        'console_scripts': [
            'wid-downloader=wid.cli.web_img_downloader:main',
            'wid-crawler=wid.cli.web_img_crawler:main',
        ],
    },
)